@@ -26,7 +26,7 @@ rba.push(
         dtype=np.float32,
     )
 )
-rba.push(np.array([[]], dtype=np.float32))
+rba.push(np.array([], dtype=np.float32).reshape(0, 3))
 
 assert generic_len(rba) == 27, f"Expected 27 elements, got {generic_len(rba)}"
 
@@ -50,7 +50,7 @@ expected = """RaggedBuffer([
 ], '4 * var * 3 * f32)"""
 
 
-assert str(rba) == expected
+assert str(rba) == expected, str(rba)
 flattened = np.array(
     [
         [2.0, 3.0, 1.0],
@@ -110,6 +110,7 @@ rb5 = rb3 + rb4
 assert np.all(
     rb5.as_array() == np.array([[0], [3], [4], [5], [10], [15]], dtype=np.int64)
 ), f"{rb5.as_array()}"
+assert rb3 + rb4 == rb4 + rb3
 
 rb6 = RaggedBufferF32.from_flattened(
     np.array(
@@ -159,6 +160,26 @@ print("TEST 4 PASSED")
 flat = ragged_buffer.cat([flati1, flati2, flati1, flati2], dim=1).as_array().flatten()
 assert np.all(
     flat
-    == np.array([0, 1, 2, 6, 0, 1, 2, 6, 3, 7, 8, 4, 5, 3, 7, 8, 4, 5], dtype=np.int64),
+    == np.array([0, 1, 2, 6, 0, 1, 2, 6, 3, 7, 8, 3, 7, 8, 4, 5, 4, 5], dtype=np.int64),
 ), f"{flat} {ragged_buffer.cat([flati1, flati2, flati1, flati2], dim=1)}"
 print("TEST 5 PASSED")
+
+
+mask = RaggedBufferI64.from_array(np.zeros((4, 1, 1), dtype=np.int64))
+offset = RaggedBufferI64.from_flattened(
+    np.array([0, 1, 2, 3, 13, 22, 32, 41, 42, 43, 44, 45,], dtype=np.int64).reshape(
+        -1, 1,
+    ),
+    np.ones(12, dtype=np.int64),
+)
+try:
+    mask = mask + offset
+except ValueError as e:
+    pass
+else:
+    assert False, "Did not raise ValueError"
+
+
+zerofeats = RaggedBufferF32(features=0)
+zerofeats.push(np.zeros((1, 0), dtype=np.float32))
+zerofeats.push(np.zeros((0, 0), dtype=np.float32))
